//
// $Id: ipfw-gtk.c,v 1.1 2001/10/12 10:33:09 mavetju Exp $
//

//
// These are the GTK-related functions to build a window, to resize
// it and pressing of a button.
//

#include <gtk/gtk.h>
#include "ipfw-graph.h"

GdkPixmap *	drawing_pixmap=NULL;
GdkPixmap *	bytes_pixmap=NULL;
GtkWidget *	drawing_area;
GtkWidget *	bytes_area;
GtkWidget *	text_area;
GdkGC *		gc_white;


//
// Get a new colour for the colourmap.
// Usage: GetPen(NewColour(red,green,blue))
//
// Credits: found on the Internet somewhere
//
GdkGC *GetPen(GdkColor *c) {
    GdkGC *gc;

    gc=gdk_gc_new(drawing_pixmap);
    gdk_gc_set_foreground(gc,c);
    return gc;
}

GdkColor *NewColour(long red,long green,long blue) {
    GdkColor *c=(GdkColor *)g_malloc(sizeof(GdkColor));

    c->red=red;
    c->green=green;
    c->blue=blue;

    gdk_color_alloc(gdk_colormap_get_system(),c);
    return c;
}


//
// Called after a resize of the window
//
/* Create a new backing pixmap of the appropriate size */
gint configure_bytes_event(GtkWidget *widget,GdkEventConfigure *event) {
    if (bytes_pixmap)
	gdk_pixmap_unref(bytes_pixmap);

    ysize_bytes=bytes_area->allocation.height;
    if (data_size!=0) {
	if (bytes_area->allocation.width>MAXSTORED)
	    xsize=MAXSTORED;
	else
	    xsize=bytes_area->allocation.width;
    }

    bytes_pixmap = gdk_pixmap_new(widget->window,
			    widget->allocation.width,
			    widget->allocation.height,
			    -1);
    gdk_draw_rectangle (bytes_pixmap,
			widget->style->white_gc,
			TRUE,
			0, 0,
			widget->allocation.width,
			widget->allocation.height);
    (*drawtable[drawtableindex].bgbytes_fun)();
    (*drawtable[drawtableindex].bytes_fun)();
    return TRUE;
}

gint configure_drawing_event(GtkWidget *widget,GdkEventConfigure *event) {
    if (drawing_pixmap)
	gdk_pixmap_unref(drawing_pixmap);

    drawing_pixmap = gdk_pixmap_new(widget->window,
			    widget->allocation.width,
			    widget->allocation.height,
			    -1);
    gdk_draw_rectangle (drawing_pixmap,
			widget->style->white_gc,
			TRUE,
			0, 0,
			widget->allocation.width,
			widget->allocation.height);
    (*drawtable[drawtableindex].bgdrawing_fun)();
    (*drawtable[drawtableindex].drawing_fun)();
    return TRUE;
}

//
// Redraw the screen from the backing pixmap (???)
//
gint expose_bytes_event (GtkWidget *widget, GdkEventExpose *event) {
    gdk_draw_pixmap(widget->window,
		    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		    bytes_pixmap,
		    event->area.x, event->area.y,
		    event->area.x, event->area.y,
		    event->area.width, event->area.height);
    return FALSE;
}

gint expose_drawing_event (GtkWidget *widget, GdkEventExpose *event) {
    gdk_draw_pixmap(widget->window,
		    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		    drawing_pixmap,
		    event->area.x, event->area.y,
		    event->area.x, event->area.y,
		    event->area.width, event->area.height);
    return FALSE;
}

//
// Button press handling
//
void print_button_press (guint32 deviceid) {
    drawtableindex++;
    if (drawtable[drawtableindex].drawing_fun==NULL) drawtableindex=0;
    (*drawtable[drawtableindex].bgbytes_fun)();
    (*drawtable[drawtableindex].bytes_fun)();
    (*drawtable[drawtableindex].bgdrawing_fun)();
    (*drawtable[drawtableindex].drawing_fun)();
}

gint button_press_event (GtkWidget *widget, GdkEventButton *event) {
    print_button_press (event->deviceid);
    return TRUE;
}

//
// Input dialog (???)
//
void input_dialog_destroy (GtkWidget *w, gpointer data) {
    *((GtkWidget **)data) = NULL;
}

void create_input_dialog(void) {
    static GtkWidget *inputd = NULL;

    if (!inputd) {
	inputd = gtk_input_dialog_new();

	gtk_signal_connect (GTK_OBJECT(inputd), "destroy",
			    (GtkSignalFunc)input_dialog_destroy, &inputd);
	gtk_signal_connect_object (GTK_OBJECT(GTK_INPUT_DIALOG(inputd)->close_button),
				"clicked",
				(GtkSignalFunc)gtk_widget_hide,
				GTK_OBJECT(inputd));
	gtk_widget_hide ( GTK_INPUT_DIALOG(inputd)->save_button);

	gtk_widget_show (inputd);
    } else {
	if (!GTK_WIDGET_MAPPED(inputd))
	    gtk_widget_show(inputd);
	else
	    gdk_window_raise(inputd->window);
    }
}

//
// Quit button
//
void quit(void) {
    gtk_exit(0);
}

//
// Initialize the GTK system
//
void init_gtk(int *argc,char **argv[]) {
    GtkWidget *window;
    GtkWidget *vbox;

    GtkWidget *button;
    GtkStyle *style;

    gtk_init(argc,argv);

    window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_name(window,"Test Input");

    vbox=gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(window),vbox);
    gtk_widget_show(vbox);

    gtk_signal_connect(GTK_OBJECT(window),"destroy",GTK_SIGNAL_FUNC(quit),NULL);

    /* Create the drawing area */
    drawing_area=gtk_drawing_area_new();
    gtk_drawing_area_size(GTK_DRAWING_AREA(drawing_area),200,300);
    gtk_box_pack_start(GTK_BOX(vbox),drawing_area,TRUE,TRUE,0);
    gtk_widget_show(drawing_area);

    bytes_area=gtk_drawing_area_new();
    gtk_drawing_area_size(GTK_DRAWING_AREA(bytes_area),200,50);
    gtk_box_pack_start(GTK_BOX(vbox),bytes_area,TRUE,TRUE,0);
    gtk_widget_show(bytes_area);

    text_area=gtk_label_new("blaat");
    gtk_box_pack_start(GTK_BOX(vbox),text_area,FALSE,FALSE,0);
    gtk_widget_show(text_area);

    /* Signals used to handle backing pixmap */
    gtk_signal_connect(GTK_OBJECT(drawing_area),"expose_event",
			(GtkSignalFunc)expose_drawing_event,NULL);
    gtk_signal_connect(GTK_OBJECT(bytes_area),"expose_event",
			(GtkSignalFunc)expose_bytes_event,NULL);
    gtk_signal_connect(GTK_OBJECT(drawing_area),"configure_event",
			(GtkSignalFunc)configure_drawing_event,NULL);
    gtk_signal_connect(GTK_OBJECT(bytes_area),"configure_event",
			(GtkSignalFunc)configure_bytes_event,NULL);

    /* Event signals */
    gtk_signal_connect(GTK_OBJECT(drawing_area),"button_press_event",
			(GtkSignalFunc)button_press_event,NULL);
    gtk_signal_connect(GTK_OBJECT(bytes_area),"button_press_event",
			(GtkSignalFunc)button_press_event,NULL);

    gtk_widget_set_events(drawing_area,   GDK_EXPOSURE_MASK
					| GDK_LEAVE_NOTIFY_MASK
					| GDK_BUTTON_PRESS_MASK
					| GDK_POINTER_MOTION_MASK
					| GDK_POINTER_MOTION_HINT_MASK);

    /* The following call enables tracking and processing of extension
	events for the drawing area */
    gtk_widget_set_extension_events(drawing_area,GDK_EXTENSION_EVENTS_CURSOR);

    /* .. And some buttons */
    button=gtk_button_new_with_label("Quit");
    gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);

    gtk_signal_connect_object(GTK_OBJECT(button),"clicked",
				GTK_SIGNAL_FUNC(gtk_widget_destroy),
				GTK_OBJECT(window));
    gtk_widget_show(button);
    gtk_widget_show(window);

    style=gtk_widget_get_default_style();
    gc_white=GetPen(NewColour(0xffff,0xffff,0xffff));

    xsize=drawing_area->allocation.width;
    ysize_drawing=drawing_area->allocation.height;
    ysize_bytes=bytes_area->allocation.height;
}
